/**************************************************************************
* FUNCTION: ALTIMPOLETIDE
* Purpose:
* To compute the pole tide displacement of the surface of the Earth for
* satellite altimetry. Only radial component of displacement is computed.
*
* This displacement is the sum total of:
*   1) The radial body pole tide of the crust of the Earth
*   2) The ocean pole tide with respect to the crust of the Earth
*   3) The radial ocean load pole tide of the crust of the Earth relative
*      to the center of mass of the total Earth system, including the
*      oceans.
*
* NOTE: Required Earth constants are defined at in the header file
*       altimpoletide.h. Changing any of them requires recompiling the code
*
* Inputs:
*   lon         - Longitude (degrees)
*   lat         - Latitude  (degrees)
*   tmjd        - Mean Julian Date (days) (e.g., Jan 1, 2000, 00:00:00 = 51544.0 )
*   xpole       - Reported X pole along the Greenwich meridian (arcsec)
*   ypole       - Reported Y pole along the 90 degree WEST meridian (arcsec)
*   modelfile[] - Full path to file containing model for ocean and load pole tide
*   itidemodel  - Flag indicating which ocean and load model to use
*                 NOTE: The body pole tide is identical for each model
*                 0 = Legacy Topex model ignoring self-consistency and load effects
*                     Load tide is zero in this case
*                 1 = Self-consistent ocean pole tide, corresponding load,
*                     and geocenter (Desai et al., 2015)
*   imeanpole   - Flag indicating the model to use for the mean pole location
*                 0 = Legacy Topex model
*                 1 = GIA mean pole (see Desai et al., 2015)
*                 2 = old IERS 2010 model
*                 3 = From Ries and Desai (2017), and adopted by IERS in 2018.
*                     The preferred model for the mean pole location is the Ries and Desai (2017) model
*                     with imeanpole = 3.
*                
* Examples:
*
* RECOMMENDED APPROACH: Use of the Desai et al. (2015) pole tide model with the Ries and Desai (2017) mean pole location
* model will have:
* itidemodel = 1, imeanpole = 3.
*
* Duplication of current Topex/Jason-series pole tide model will have:
* itidemodel = 0, imeanpole = 0.
*
* Use of the Desai et al. (2015) pole tide model with their recommended mean pole location
* model will have:
* itidemodel = 1, imeanpole = 1.
*
* Use of the Desai et al. (2015) pole tide model with the old IERS2010 mean pole location
* model will have:
* itidemodel = 1, imeanpole = 2.
*
* Output:
*   poletide[3] - Index 0 = Body (Solid Earth) pole tide (m)
*                 Index 1 = Ocean pole tide with respect to crust (m)
*                 Index 2 = Ocean load pole tide (radial) with respect to
*                           center of mass of total Earth system (m)
*   m12[2]      - Wobble parameters in arcseconds
*                 m12[0] =  (xpole - x_mean)
*                 m12[1] = -(ypole - y_mean)
*                 x_mean = Model for mean reported pole location
*                 y_mean = Model for mean reported pole location
*
* Author:
* Shailen Desai
* Date: 03/09/2021
*
***************************************************************************/
#include "altimpoletide.h"

int altimpoletide(double poletide[3], double m12[2], double lon, double lat, 
                  double tmjd, double xpole, double ypole, char modelfile[],
                  char itidemodel, char imeanpole)
{
   double latr, lonr;                        /* Latitude and longitude in radians */
   double xm1, xm2;                          /* Admittance for body pole tide     */
   double oceanm1, oceanm2;                  /* Admittance for ocean pole tide    */
   double rloadm1, rloadm2;                  /* Admittance for load pole tide     */
   double xmean, ymean;                      /* Mean pole locations (arcsec)      */
   double wlon, wlat, mapinterp[4];          /* Map interpolation parameters      */
   double k2[2], h2[2], Gamma[2];            /* Love numbers                      */
   int32_t         i1, i2, j1, j2, k;
   static double   Amp_Pole, Amp_Pole_Ocean; /* Amplitude factors (m/arcsec)      */
   static double   map[MAXLON][MAXLAT][4];   /* Map of ocean and load pole tide   */
   static double   Lon0, Lat0, Dlon, Dlat;   /* Map grid parameters               */
   static int32_t  Nlon, Nlat;               /* Map grid parameters               */
   static char     ifirst = 0;

/* On first call read the file containing the model for the ocean and load pole tide
*  and compute amplitute scale factors in m/arcsec
*/
   if (ifirst == 0) {
     ifirst = 1;

/* Read model file
*/
     readmodelfile(map, &Lon0, &Lat0, &Dlon, &Dlat, &Nlon, &Nlat, modelfile);

/* Compute pole tide scale factors, Amp_Pole and Amp_Pole_Ocean in units of m/arcsec
*  First compute Amp_Pole in km/radian
*  Then convert to m/arcsec
*/
     Amp_Pole       = (OMEGA0*REARTH*REARTH)*(OMEGA0*REARTH*REARTH)/GMEARTH;
     Amp_Pole       = 1.0e3*Amp_Pole*(PI/180.0)/3600.0;
     Amp_Pole_Ocean = sqrt(8.0*PI/15.0)*Amp_Pole;

     fprintf(stdout,"altimpoletide: Pole Tide Amplitude (m/arcsec)                    : %12.6f\n", Amp_Pole);
     fprintf(stdout,"altimpoletide: Ocean Pole Tide Amplitude (m/arcsec)              : %12.6f\n", Amp_Pole_Ocean);
     fprintf(stdout,"altimpoletide: Done reading Desai ocean/load pole tide model map : %s\n", modelfile);

   }

/* Convert latitude and longitude from degrees to radians
*/
   latr = lat*(PI/180.0);
   lonr = lon*(PI/180.0);

/* Compute the mean pole location
*/
   computemeanpole(&xmean, &ymean, tmjd, imeanpole);

/* Compute wobble parameters in arcsec
*/
   m12[0] =   xpole - xmean;
   m12[1] = -(ypole - ymean);

/* Define Body Love Numbers appropriate for pole tide
*/
   if (itidemodel == 0) {
     k2[0]    = K2_REALTP;
     k2[1]    = K2_IMAGTP;
     h2[0]    = H2_REALTP;
     h2[1]    = H2_IMAGTP;
   }
   else if (itidemodel == 1) {
     k2[0]    = K2_REAL;
     k2[1]    = K2_IMAG;
     h2[0]    = H2_REAL;
     h2[1]    = H2_IMAG;
   }
   else {
     fprintf(stderr,"altimpoletide: Unrecognized pole tide model type %d\n", itidemodel);
     exit(1);
   }
   Gamma[0] = 1.0 + k2[0] - h2[0];
   Gamma[1] =       k2[1] - h2[1];

/* Define grid points and weights for longitude interpolation of map
*/
   if (lon < Lon0) {
     lon = lon + 360.0;
   }
   i1 = (int32_t) ((lon - Lon0)/Dlon);
   i2 = i1 + 1;
   if (i2 >= Nlon) {
     i2 = i2 - Nlon;
   }
   wlon = (lon - (Lon0 + i1*Dlon))/Dlon;
  
/* Define grid points and weights for latitude interpolation of map
*  If latitude out of range, set to extreme values
*/
   if (lat < Lat0) {
     lat = Lat0;
   }
   j1    = (int32_t) ((lat - Lat0)/Dlat);
   j2    = j1 + 1;
   if (j2 >= Nlat) {
     j2 = Nlat - 1;
   }
   wlat = (lat - (Lat0 + j1*Dlat))/Dlat;

/* Billinear interpolation of grid 
*/
   for (k = 0; k < 4; ++k) {
     mapinterp[k] = 0.0;
     mapinterp[k] += (1.0-wlon)*(1.0-wlat)*map[i1][j1][k];
     mapinterp[k] += (    wlon)*(1.0-wlat)*map[i2][j1][k];
     mapinterp[k] += (1.0-wlon)*(    wlat)*map[i1][j2][k];
     mapinterp[k] += (    wlon)*(    wlat)*map[i2][j2][k];
   }

/* Compute body pole tide, ocean pole tide and radial load pole tide w.r.t. CM
*/
   if (itidemodel == 0) {
     xm1         = -Amp_Pole*sin(latr)*cos(latr)*(h2[0]*cos(lonr));
     xm2         = -Amp_Pole*sin(latr)*cos(latr)*(h2[0]*sin(lonr));
     if ((fabs(mapinterp[0]) < 1.0e-6) && (fabs(mapinterp[1]) < 1.0e-6)) {
       oceanm1   = 0.0;
       oceanm2   = 0.0;
     }
     else {
       oceanm1     = -Amp_Pole*sin(latr)*cos(latr)*(Gamma[0]*cos(lonr));
       oceanm2     = -Amp_Pole*sin(latr)*cos(latr)*(Gamma[0]*sin(lonr));
     }

     poletide[0] =     xm1*m12[0] +     xm2*m12[1];
     poletide[1] = oceanm1*m12[0] + oceanm2*m12[1];
     poletide[2] = 0.0;
   }
   else if (itidemodel == 1) {
     xm1     = -Amp_Pole*sin(latr)*cos(latr)*(h2[0]*cos(lonr) - h2[1]*sin(lonr));
     xm2     = -Amp_Pole*sin(latr)*cos(latr)*(h2[0]*sin(lonr) + h2[1]*cos(lonr));
     oceanm1 = Amp_Pole_Ocean*(Gamma[0]*mapinterp[0] - Gamma[1]*mapinterp[1]);
     oceanm2 = Amp_Pole_Ocean*(Gamma[0]*mapinterp[1] + Gamma[1]*mapinterp[0]);
     rloadm1 = Amp_Pole_Ocean*(Gamma[0]*mapinterp[2] - Gamma[1]*mapinterp[3]);
     rloadm2 = Amp_Pole_Ocean*(Gamma[0]*mapinterp[3] + Gamma[1]*mapinterp[2]);

     poletide[0] =     xm1*m12[0] +     xm2*m12[1];
     poletide[1] = oceanm1*m12[0] + oceanm2*m12[1];
     poletide[2] = rloadm1*m12[0] + rloadm2*m12[1];
   }
   else {
     fprintf(stderr,"altimpoletide: Unrecognized pole tide model type %d\n", itidemodel);
     exit(1);
   }

   return 0;
}
/**************************************************************************
* FUNCTION: COMPUTEMEANPOLE
* Purpose:
* To compute the mean pole location in arcseconds
* The recommended approach uses imodel = 3.
*
* Input:
*     tmjd   - Time in MJD (e.g., Jan 1, 2000, 00:00:00 = 51544.0)
*     imodel - Flag indicating which model to use for mean pole
*              0 = Topex altimetry model
*              1 = GIA model from Desai et al., (2015)
*              2 = IERS2010 model
*              3 = From Ries and Desai (2017), and adopted by IERS in 2018.
*                  The preferred model for the mean pole location is the Ries and Desai (2017) model
*                  with imeanpole = 3.
*
* Output:
*     xmean  - Mean X pole along the Greenwich meridian (arcsec)
*     ymean  - Mean Y pole along the 90 degree WEST meridian  (arcsec)
*
**************************************************************************/
int computemeanpole(double *xmean, double *ymean, double tmjd, char imodel)
{
   double dt;                          /* Years since reference, MJD2000        */
   double x[NMEANPOLE], y[NMEANPOLE];  /* Coefficients for mean pole tide model */
   int32_t  k;

/* Define cubic polynomial coefficients in arcsec/yr^i;
*/
   if (imodel == 0) {
       x[0] = 0.042;
       x[1] = 0.0;
       x[2] = 0.0;
       x[3] = 0.0;
       y[0] = 0.293;
       y[1] = 0.0;
       y[2] = 0.0;
       y[3] = 0.0;
   }
   else if (imodel == 1) {
       x[0] = 0.05097;
       x[1] = 0.00062;
       x[2] = 0.0;
       x[3] = 0.0;
       y[0] = 0.33449;
       y[1] = 0.00348;
       y[2] = 0.0;
       y[3] = 0.0;
   }
   else if (imodel == 2) {
     if (tmjd <= 55197.0) {
       x[0] =  55.974e-3;
       x[1] =   1.8243e-3;
       x[2] =   0.18413e-3;
       x[3] =   0.007024e-3;
       y[0] = 346.346e-3;
       y[1] =   1.7896e-3;
       y[2] =  -0.10729e-3;
       y[3] =  -0.000908e-3;
     }
     else {
       x[0] =  23.513e-3;
       x[1] =   7.6141e-3;
       x[2] =   0.0;
       x[3] =   0.0;
       y[0] = 358.891e-3;
       y[1] =  -0.6287e-3;
       y[2] =   0.0;
       y[3] =   0.0;
     }
   }
   else if (imodel == 3) {
       x[0] = 0.055000;
       x[1] = 0.001677;
       x[2] = 0.0;
       x[3] = 0.0;
       y[0] = 0.320500;
       y[1] = 0.003460;
       y[2] = 0.0;
       y[3] = 0.0;
   }
   else {
     fprintf(stderr,"computemeanpole: Unrecognized mean pole model type %d\n", imodel);
     exit(1);
   }
   
   dt     = (tmjd - MJD2000)/365.25;
   *xmean = x[NMEANPOLE-1];
   *ymean = y[NMEANPOLE-1];
   for (k = NMEANPOLE-2; k >= 0; --k) {
     *xmean = x[k] + (*xmean)*dt;
     *ymean = y[k] + (*ymean)*dt;
   }

   return 0;
}
/**************************************************************************
* FUNCTION: READMODELFILE
* Purpose:
* To read the file containing coefficients that define the ocean and radial
* load pole tide.
*
* Input:
*    modelfile[]    - Full path to file containing model for ocean and 
*                     radial load pole tide
* Output:
*    map[MAXLON][MAXLAT][4] - Map of ocean pole tide and radial load pole tide
*                             Returned coefficients are:
*                             Ocean(real) (dimensionless)
*                             Ocean(imag) (dimensionless)
*                             Load(real) (dimensionless)
*                             Load(imag) (dimensionless)
*    Lon0                   - Longitude (degrees) of west-most grid point
*    Lat0                   - Latitude (degrees) of south-most grid point
*    Dlon                   - Longitude step (degrees)
*    Dlat                   - Latitude step (degrees)
*    Nlon                   - Number of longitude nodes
*    Nlat                   - Number of latitude nodes
**************************************************************************/
int readmodelfile(double map[MAXLON][MAXLAT][4], double *Lon0, double *Lat0,
                  double *Dlon, double *Dlat, int32_t *Nlon, int32_t *Nlat,
                  char modelfile[])
{
   double oceanr, oceani; /* Real and imaginary coefficients for ocean pole tide */
   double rloadr, rloadi; /* Real and imaginary coefficients for load  pole tide */
   int32_t i, j, k;       /* Index for longitude, latitude, and parameter        */
   int32_t datalinecount; /* Counter for lines in file                           */
   char line[MAXCHAR];    /* Line from input file                                */
   char *p_sharp;         /* Pointer to # in line                                */
   FILE *ifp;             /* Pointer to file                                     */

/* Initialize the map to zeros
*/
   *Lon0 = 0.0;
   *Lat0 = 0.0;
   *Dlat = 0.0;
   *Dlon = 0.0;
   *Nlon = 0;
   *Nlat = 0;
   for (i = 0; i < MAXLON; ++i) {
     for (j = 0; j < MAXLAT; ++j) {
       for (k = 0; k < 4; ++k) {
         map[i][j][k] = 0.0;
       }
     }
   }

/* Open file
*/
   ifp = fopen(modelfile,"r");
   if (ifp == NULL) {
     fprintf(stdout,"readmodelfile: ERROR - Unable to open file %s\n", modelfile);
     exit(1);
   }

/* Read the file
*  Ignore all lines starting with #
*  Ignore any part of line following #
*  First data line is head info with longitude and latitude of south-west
*  node of map, longitude and latitude step, and number of longitude and
*  latitude nodes
*/
   datalinecount = 0;
   while (fgets(line,MAXCHAR,ifp) != NULL) {
     if (*line == '#') continue;
     if ( (p_sharp = index(line,'#') ) ) { *p_sharp = '\0'; }
     if (datalinecount == 0) {
       if (strstr(line,"Info:") == NULL) {
         fprintf(stdout,"readmodelfile: ERROR - First data line in input file (%s) must start with Info:.\n", modelfile);
         fprintf(stdout,"readmodelfile:         Info line provides map grid information.\n");
         exit(1);
       }
       if (sscanf(line,"Info: %lf %lf %lf %lf %d %d", Lon0, Lat0, Dlon, Dlat, Nlon, Nlat) == 6) {
         if (*Nlon > MAXLON) {
           fprintf(stdout,"readmodelfile: ERROR - Input file %s has %d longitude nodes.\n", modelfile, *Nlon);
           fprintf(stdout,"readmodelfile:         Only %d longitude nodes allowed.\n", MAXLON);
           exit(1);
         }
         if (*Nlat > MAXLAT) {
           fprintf(stdout,"readmodelfile: ERROR - Input file %s has %d latitude nodes.\n", modelfile, *Nlat);
           fprintf(stdout,"readmodelfile:         Only %d latitude nodes allowed.\n", MAXLAT);
           exit(1);
         }
         datalinecount++;
       }
       else {
         fprintf(stdout,"readmodelfile: ERROR - Unable to read Info line in input file %s.\n", modelfile);
         exit(1);
       }
     }
     else {
       if (sscanf(line,"%d %d %lf %lf %lf %lf", &i, &j, &oceanr, &oceani, &rloadr, &rloadi) == 6) {
         datalinecount++;
         map[i][j][0] = oceanr;
         map[i][j][1] = oceani;
         map[i][j][2] = rloadr;
         map[i][j][3] = rloadi;
       }
     }
   }

/* Check that whole map was defined with the header line
*/
   if (datalinecount != (*Nlon)*(*Nlat)+1) {
     fprintf(stdout,"readmodelfile: ERROR - Only found %d of %d expected data lines in file %s\n", datalinecount, (*Nlon)*(*Nlat)+1, modelfile);
     exit(1);
   }

/* Close model file
*/
   fclose(ifp);

/* Log the various parameters
*/
   fprintf(stdout,"readmodelfile: Using ocean pole tide model from     : %s\n", modelfile);
   fprintf(stdout,"readmodelfile: Longitude Grid (First/Step/Nodes)    : %9.2f  %9.2f  %6d\n", *Lon0, *Dlon, *Nlon);
   fprintf(stdout,"readmodelfile: Latitude  Grid (First/Step/Nodes)    : %9.2f  %9.2f  %6d\n", *Lat0, *Dlat, *Nlat);
   fprintf(stdout,"readmodelfile: %d of expected %d data lines found\n", datalinecount, (*Nlon)*(*Nlat)+1);

   return 0;
}
