/**************************************************************************************
* FUNCTION: GENALTIMPOLETIDE
* Purpose:
* To compute the radial body, ocean, and ocean load pole tide displacement of the
* surface of the Earth. Over land, or inland seas, the ocean pole tide will be zero.
*
* Required external inputs are:
* 1. Observed X and Y pole locations in radians
* 2. Model for the mean X and Y pole locations in radians
* 3. Real and Imaginary Parts of Body Love Numbers, h2, and k2 appropriate for the pole
*    tide 
*
* Author:
* Shailen Desai
* Date: 04/02/2015
*
**************************************************************************************/
#include "altimpoletide.h"

int main(int argc, char *argv[])
{
   double poletide[3]; /* Index 0 = Body (Solid Earth) Pole Tide          (m)            */
                       /* Index 1 = Ocean Pole Tide with respect to crust (m)            */
                       /* Index 2 = Ocean Load Pole Tide with respect to                 */
                       /*           center of mass of total Earth system  (m)            */
   double m12[2];      /* Wobble parameters                                              */
   double lon;         /* Longitude (degrees)                                            */
   double lat;         /* Latitude (degrees)                                             */
   double tmjd;        /* Mean Julian Date (days) (e.g., Jan 1, 2000 00:00:00 = 51544.0  */
   double xpole;       /* Reported pole location (arcseconds), along Greenwich meridian  */
   double ypole;       /* Reported pole location (arcseconds), along 90 degrees WEST     */
   double total_poletide_tp; /* Total radial pole tide (mm) using Topex model            */
   double total_poletide;    /* Total radial pole tide (mm) using Desai (2015) model     */
                             /* with Desai et al (2015) mean pole                        */
   double total_poletide_lin; /* Total radial pole tide (mm) using Desai (2015) model    */
                              /* with Ries and Desai (2017) mean pole (RECOMMENDED)      */
   FILE   *ifp;            /* Pointer to reported pole location file                     */
   double refpoletide_tp;  /* Reference Topex pole tide from validation file             */
   double refpoletide;     /* Reference Desai pole tide with Desai mean pole             */
                           /* from validation file                                       */
   double refpoletide_lin; /* Reference Desai pole tide with Ries/Desai mean pole        */
                           /* from validation file                                       */
   double diffpoletide_tp; /* Difference between computed and reference Topex pole tide  */
   double diffpoletide;    /* Difference between computed and reference Desai pole tide  */
                           /* with Desai mean pole                                       */
   double diffpoletide_lin;/* Difference between computed and reference Desai pole tide  
                           /* with Ries/Desai mean pole                                  */

/* Loop through every entry of validation file
*/
   if ((ifp = fopen("data/altimpoletide_validation.txt", "r")) == NULL) {
     fprintf(stdout,"Cannot find validation file: data/altimpoletide_validation.txt\n");
     exit(1);
   }
   while (fscanf(ifp,"%lf %lf %lf %lf %lf %lf %lf %lf\n", &tmjd, &lon, &lat, &xpole, &ypole, &refpoletide_tp, &refpoletide, &refpoletide_lin) == 8) {

/* Compute Topex model, from total geocentric pole tide, and convert to mm.
*  NOTE: Ocean pole tide is defined as zero over land in the provided (Desai) model, using
*  land mask for that model.
*  Load tide is defined to be zero.
*/
     altimpoletide(poletide, m12, lon, lat, tmjd, xpole, ypole, MODELFILE, 0, 0);
     total_poletide_tp = 1.0e3*(poletide[0] + poletide[1] + poletide[2]);

/* Compute Desai pole tide model with GIA mean pole from Desai et al (2015), form total geocentric tide, and convert to mm
*  NOTE: Ocean pole tide is defined as zero over land in the provided model, using land mask
*  for that model.
*/
     altimpoletide(poletide, m12, lon, lat, tmjd, xpole, ypole, MODELFILE, 1, 1);
     total_poletide = 1.0e3*(poletide[0] + poletide[1] + poletide[2]);

/* Compute Desai pole tide model with Ries and Desai (2017) mean pole, form total geocentric tide, and convert to mm
*  NOTE: Ocean pole tide is defined as zero over land in the provided model, using land mask
*  for that model.
*  NOTE: THIS IS THE RECOMMENDED APPROACH.
*/
     altimpoletide(poletide, m12, lon, lat, tmjd, xpole, ypole, MODELFILE, 1, 3);
     total_poletide_lin = 1.0e3*(poletide[0] + poletide[1] + poletide[2]);
     
     diffpoletide_tp  = total_poletide_tp  - refpoletide_tp;
     diffpoletide     = total_poletide     - refpoletide;
     diffpoletide_lin = total_poletide_lin - refpoletide_lin;
     if (fabs(diffpoletide_tp) >= 1.0e-6) {
       fprintf(stdout,"WARNING: Difference between computed and reference Topex pole tide > 1.0e-6 mm. MJD = %f\n", tmjd);
     }
     if (fabs(diffpoletide) >= 1.0e-6) {
       fprintf(stdout,"WARNING: Difference between computed and reference Desai pole tide > 1.0e-6 mm. MJD = %f\n", tmjd);
     }
     if (fabs(diffpoletide_lin) >= 1.0e-6) {
       fprintf(stdout,"WARNING: Difference between computed and reference Desai pole tide with linear mean pole > 1.0e-6 mm. MJD = %f\n", tmjd);
     }
     fprintf(stdout,"%12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f  %12.6f\n",
              tmjd, lon, lat, xpole, ypole, total_poletide_tp, total_poletide, total_poletide_lin, diffpoletide_tp, diffpoletide, diffpoletide_lin);
   }
   fclose(ifp);

   return 0;
}
